@@ -1,7 +1,9 @@
 import shlex
+import json
 
 from typing import List, Optional, Union, IO, Callable
 from collections import deque
+from http.server import BaseHTTPRequestHandler, HTTPServer
 from threading import current_thread, Event
 from time import sleep
 
@@ -16,7 +18,10 @@ from .teletypes import (
     Chat,
     ChatMember,
     BotCommand,
-    InlineQueryResult)
+    InlineQueryResult,
+    WebhookInfo,
+    InputMedia,
+    Poll)
 
 from .commands import CommandContainer
 from .util import check_args
@@ -65,7 +70,16 @@ from .api_request import (
     answer_callback_query,
     set_my_commands,
     get_my_commands,
-    answer_inline_query)
+    answer_inline_query,
+    set_webhook,
+    delete_webhook,
+    get_webhook_info,
+    edit_message_text,
+    edit_message_caption,
+    edit_message_media,
+    edit_message_reply_markup,
+    stop_poll,
+    delete_message)
 
 from ..util import Listener
 
@@ -74,6 +88,7 @@ class OrigamiBot:
     """Telegram bot class."""
     def __init__(self, token):
         self.token = token
+        self.webhook = None
         self.updates = deque()
         self.interval = 0.1
 
@@ -82,6 +97,12 @@ class OrigamiBot:
             target=self._listen_loop,
             daemon=True)
 
+        self._webhook_thread = StoppableThread(
+            name='Webhook thread',
+            target=self._webhook_loop,
+            daemon=True
+        )
+
         self._process_thread = StoppableThread(
             name='Update process thread',
             target=self._process_updates_loop,
@@ -96,7 +117,10 @@ class OrigamiBot:
 
     def start(self):
         """Start listening for updates. Non-blocking!"""
-        self._listen_thread.start()
+        if self.webhook is None:
+            self._listen_thread.start()
+        else:
+            self._webhook_thread.start()
         self._process_thread.start()
 
     def stop(self):
@@ -986,6 +1010,172 @@ class OrigamiBot:
             switch_pm_parameter
         )
 
+    def set_webhook(self,
+                    url: str,
+                    certificate: Optional[IO] = None,
+                    max_connections: Optional[int] = None,
+                    allowed_updates: Optional[List[str]] = None
+                    ) -> bool:
+        """Use this method to specify a url
+        and receive incoming updates via an outgoing webhook.
+
+        Whenever there is an update for the bot,
+        we will send an HTTPS POST request to the specified url,
+        containing a JSON-serialized Update.
+
+        Returns True on success.
+        """
+        self.webhook = url
+        return set_webhook(
+            self.token,
+            url,
+            certificate,
+            max_connections,
+            allowed_updates
+        )
+
+    def delete_webhook(self):
+        """Use this method to remove webhook integration
+        if you decide to switch back to getUpdates.
+
+        Returns True on success.
+        """
+        return delete_webhook(self.token)
+
+    def get_webhook_info(self) -> WebhookInfo:
+        """Use this method to get current webhook status.
+
+        On success, returns a WebhookInfo object.
+        If the bot is using getUpdates,
+        will return an object with the url field empty.
+        """
+        return get_webhook_info(self.token)
+
+    def edit_message_text(self,
+                          chat_id: Union[int, str],
+                          text: str,
+                          message_id: Optional[int] = None,
+                          inline_message_id: Optional[str] = None,
+                          parse_mode: Optional[str] = None,
+                          disable_web_page_preview: Optional[bool] = None,
+                          reply_markup: Optional[InlineKeyboardMarkup] = None
+                          ) -> Union[Message, bool]:
+        """Use this method to edit text and game messages.
+
+        On success, if edited message is sent by the bot,
+        the edited Message is returned,
+        otherwise True is returned.
+        """
+        return edit_message_text(
+            self.token,
+            chat_id,
+            text,
+            message_id,
+            inline_message_id,
+            parse_mode,
+            disable_web_page_preview,
+            reply_markup
+        )
+
+    def edit_message_caption(self,
+                             chat_id: Union[int, str],
+                             caption: Optional[str] = None,
+                             message_id: Optional[int] = None,
+                             inline_message_id: Optional[str] = None,
+                             parse_mode: Optional[str] = None,
+                             reply_markup: Optional[
+                                 InlineKeyboardMarkup] = None
+                             ) -> Union[Message, bool]:
+        """Use this method to edit captions of messages.
+
+        On success, if edited message is sent by the bot,
+        the edited Message is returned,
+        otherwise True is returned.
+        """
+        return edit_message_caption(
+            self.token,
+            chat_id,
+            caption,
+            message_id,
+            inline_message_id,
+            parse_mode,
+            reply_markup
+        )
+
+    def edit_message_media(self,
+                           chat_id: Union[int, str],
+                           media: InputMedia,
+                           message_id: Optional[int] = None,
+                           inline_message_id: Optional[int] = None,
+                           reply_markup: Optional[InlineKeyboardMarkup] = None
+                           ) -> Union[Message, bool]:
+        """Use this method to edit animation, audio,
+        document, photo, or video messages.
+
+        On success, if the edited message was sent by the bot,
+        the edited Message is returned,
+        otherwise True is returned.
+        """
+        return edit_message_media(
+            self.token,
+            chat_id,
+            media,
+            message_id,
+            inline_message_id,
+            reply_markup
+        )
+
+    def edit_message_reply_markup(self,
+                                  chat_id: Optional[Union[int, str]] = None,
+                                  message_id: Optional[int] = None,
+                                  inline_message_id: Optional[str] = None,
+                                  reply_markup: Optional[
+                                      InlineKeyboardMarkup] = None
+                                  ) -> Union[Message, bool]:
+        """Use this method to edit only the reply markup of messages.
+
+        On success, if edited message is sent by the bot,
+        the edited Message is returned,
+        otherwise True is returned.
+        """
+        return edit_message_reply_markup(
+            self.token,
+            chat_id,
+            message_id,
+            inline_message_id,
+            reply_markup
+        )
+
+    def stop_poll(self,
+                  chat_id: Union[int, str],
+                  message_id: int,
+                  reply_markup: Optional[InlineKeyboardMarkup] = None
+                  ) -> Poll:
+        """Use this method to stop a poll which was sent by the bot.
+
+        On success, the stopped Poll with the final results is returned.
+        """
+        return stop_poll(
+            self.token,
+            chat_id,
+            message_id,
+            reply_markup
+        )
+
+    def delete_message(self,
+                       chat_id: Union[int, str],
+                       message_id: int
+                       ) -> bool:
+        """Use this method to delete a message, including service messages
+
+        Returns True on success.
+        """
+        return delete_message(
+            self.token,
+            chat_id,
+            message_id
+        )
+
     def _process_updates_loop(self):
         """The main processing thread.
 
@@ -1085,5 +1275,23 @@ class OrigamiBot:
             method = getattr(listener, event)
             method(*args, **kwargs)
 
+    def _webhook_loop(self):
+        class HandleUpdates(BaseHTTPRequestHandler):
+            bot = self
+
+            def _set_headers(self):
+                self.send_response(200)
+                self.end_headers()
+
+            def do_POST(self):
+                '''Reads post request body'''
+                self._set_headers()
+                content_len = int(self.headers.get('content-length', 0))
+                post_body = self.rfile.read(content_len).decode()
+                update = Update(**json.loads(post_body))
+                HandleUpdates.bot.process_update(update)
+
+        HTTPServer((self.webhook, 443), HandleUpdates).serve_forever()
+
 
 __all__ = [OrigamiBot.__name__]
