@@ -357,6 +357,13 @@ class InputMedia:
     def __post_init__(self):
         self._file = None
 
+    def get_data_n_files(self):
+        data = asdict(self)
+        attach, file = self.file
+        if self.file is None:
+            return data, {}
+        return data, {attach: file}
+
     @property
     def file(self):
         """Returns local file attach://[name], and [file path]
@@ -902,6 +909,17 @@ class InlineQueryResultCachedAudio(InlineQueryResult):
     input_message_content: InputMessageContent = None
 
 
+@dataclass
+class WebhookInfo:
+    url: str
+    has_custom_certificate: bool
+    pending_update_count: int
+    last_error_date: int = None
+    last_error_message: str = None
+    max_connections: int = None
+    allowed_update: list = None
+
+
 # Collect all dataclasses in this module
 api_types = [
     i[1] for i in getmembers(modules[__name__],
@@ -961,7 +979,9 @@ name_type_map = {
 
 
 def asdict(o):
-    return {k: v
+    if o is None:
+        return None
+    return {k: asdict(v) if is_dataclass(v) else v
             for k, v in o.__dict__.items()
             if v is not None and not k.startswith('_')}
 
